@@ -52,9 +52,9 @@ bool match_valsStr(const char *lval, enum Operator op, const char *rval) {
 		ret = true;
 	else if (op == neq && cmp != 0)
 		ret = true;
-	else if (op == lt && cmp <= 0)
+	else if (op == le && cmp <= 0)
 		ret = true;
-	else if (op == gt && cmp >= 0)
+	else if (op == ge && cmp >= 0)
 		ret = true;
 	else if (op == l && cmp < 0)
 		ret = true;
@@ -74,9 +74,9 @@ bool match_valsInt(int lval, enum Operator op, int rval) {
 		ret = true;
 	else if (op == neq && lval != rval)
 		ret = true;
-	else if (op == lt && lval <= rval)
+	else if (op == le && lval <= rval)
 		ret = true;
-	else if (op == gt && lval >= rval)
+	else if (op == ge && lval >= rval)
 		ret = true;
 	else if (op == l && lval < rval)
 		ret = true;
@@ -86,36 +86,137 @@ bool match_valsInt(int lval, enum Operator op, int rval) {
 	return ret;
 }
 
-bool match_vals(const char *lvalStr, enum Operator op, const char *rvalStr) {
+bool match_vals_devpath(const char *lvalStr, enum Operator op, const char *rvalStr, enum Valuetype lvalType, enum Valuetype rvalType) {
 	bool ret = false;
+	int comp = 0;
+
+	int lvalLen = strlen(lvalStr);
+	int rvalLen = strlen(rvalStr);
+	char *lval = calloc(lvalLen, sizeof(char));
+	char *rval = calloc(rvalLen, sizeof(char));
+
+		if(lval && rval) {
+		char *lStr = NULL;
+		char *rStr = NULL;
+		char *lvalAllocPtr = lval;
+		char *rvalAllocPtr = rval;
+		strcpy(lval, lvalStr);
+		strcpy(rval, rvalStr);
+
+		// compare devnum like a software version number using sub version (e. g. major, minor, patch version)
+		while ((lStr = strsep(&lval, ".")) && (rStr = strsep(&rval, ".")))
+		{
+			if (match_vals(lStr, g, rStr, lvalType, rvalType)) {
+				comp = 1;
+				break;
+			} else if (match_vals(lStr, l, rStr, lvalType, rvalType)) {
+				comp = -1;
+				break;
+			}
+		}
+
+		// compare result -1, 0, 1 with given operator
+		ret = match_valsInt(comp, op, 0);
+
+		free(lvalAllocPtr);
+		free(rvalAllocPtr);
+	}
+
+	return ret;
+}
+
+bool match_vals_devpath_autotype(const char *lvalStr, enum Operator op, const char *rvalStr, enum Valuetype lvalType) {
+	return match_vals_devpath(lvalStr, op, rvalStr, lvalType, UNKNOWN);
+}
+
+bool match_vals(const char *lvalStr, enum Operator op, const char *rvalStr, enum Valuetype lvalType, enum Valuetype rvalType) {
+	bool ret = false;
+	bool useDevpathMatching = false;
 	char* lend = NULL;
 	char* rend = NULL;
-	int lval = strtol(lvalStr, &lend, 16);
-	int rval = strtol(rvalStr, &rend, 16);
+	char* tmpStr = NULL;
+	int lbase = 16;
+	int rbase = 16;
+	int lval = -1;
+	int rval = -1;
+	int rvalLen = strlen(rvalStr);
 
-	if (lend && *lend != 0)
-		lval = -1;
+	if (rvalType == UNKNOWN) {
+		if (rvalLen >= 2 && rvalStr[0] == '"' && rvalStr[rvalLen-1] == '"') { // right value type is STRING
+			rvalType = STRING;
+			rvalLen -= 2;
 
-	if (rend && *rend != 0)
-		rval = -1;
+			tmpStr = calloc(rvalLen + 1, sizeof(char));
+			strncpy(tmpStr, rvalStr + 2, rvalLen);
+			tmpStr[rvalLen] = '\0';
+			rvalStr = tmpStr;
+		} else if (rvalLen >= 2 && rvalStr[0] == '\\') { // right value type is HEX, DEC or UNKNOWN
+			if (rvalStr[1] == 'x') {
+				rvalType = HEX;
+				rbase = 16;
+			} else if (rvalStr[1] == 'd') {
+				rvalType = DEC;
+				rbase = 10;
+			}
 
-	if (lval != -1 && rval != -1)
+			if (rvalType != UNKNOWN) { // right value type is HEX or DEC
+				rvalLen -= 2;
+				tmpStr = calloc(rvalLen + 1, sizeof(char));
+				strncpy(tmpStr, rvalStr + 2, rvalLen);
+				tmpStr[rvalLen] = '\0';
+				rvalStr = tmpStr;
+			}
+		}
+	}
+
+	// use (determined) rbase for lbase as default
+	// will be overwritten if proper lbase is given
+	lbase = rbase;
+
+	if (lvalType == HEX)
+		lbase = 16;
+	else if (lvalType == DEC)
+		lbase = 10;
+
+	if (rvalType != STRING) {
+		if (strchr(rvalStr, '.')) // right value type contains '.'
+			useDevpathMatching = true;
+		else { // right value type is HEX, DEC or UNKNOWN
+			lval = strtol(lvalStr, &lend, lbase);
+			rval = strtol(rvalStr, &rend, rbase);
+
+			if (lend && *lend != 0)
+				lval = -1;
+
+			if (rend && *rend != 0)
+				rval = -1;
+		}
+	}
+
+	if (useDevpathMatching) // use devpath match mode for right value type
+		ret = match_vals_devpath(lvalStr, op, rvalStr, lvalType, rvalType);
+	else if (lval != -1 && rval != -1) // integer match of right value type
 		ret = match_valsInt(lval, op, rval);
-	else
+	else if (rvalType == STRING || rvalType == UNKNOWN) // string match of right value type
 		ret = match_valsStr(lvalStr, op, rvalStr);
 
 	if (debuglog)
 		syslog(LOG_DEBUG, "match_vals:%i (%s %s %s), (%i %s %i)\n", ret, lvalStr, usbauth_op_to_str(op), rvalStr, lval, usbauth_op_to_str(op), rval);
 
+	if (tmpStr)
+		free(tmpStr);
+
 	return ret;
 }
 
+bool match_vals_autotype(const char *lvalStr, enum Operator op, const char *rvalStr, enum Valuetype lvalType) {
+	return match_vals(lvalStr, op, rvalStr, lvalType, UNKNOWN);
+}
+
 bool match_vals_interface(struct Auth *rule, struct Data *d, struct udev_device *interface) {
 	bool ret = false;
-	char* tmpStr = NULL;
 	const char* lvalStr = NULL;
 	const char* rvalStr = d->val;
-	int rvalLen = 0;
 	const char* type = udev_device_get_devtype(interface);
 	char cntStr[16];
 	strcpy(cntStr, "");
@@ -123,16 +224,6 @@ bool match_vals_interface(struct Auth *rule, struct Data *d, struct udev_device
 	if (!type || !rvalStr || strcmp(type, "usb_interface") != 0)
 		return false;
 
-	rvalLen = strlen(rvalStr);
-	if (rvalLen >= 2 && rvalStr[0] == '"' && rvalStr[rvalLen-1] == '"')
-	{
-		rvalLen -= 2;
-		tmpStr = calloc(rvalLen + 1, sizeof(char));
-		strncpy(tmpStr, rvalStr + 1, rvalLen);
-		tmpStr[rvalLen] = '\0';
-		rvalStr = tmpStr;
-	}
-
 	if (intfcount == d->param) { // intfcount parameter is not in sysfs
 		char* rend = NULL;
 		int rval = strtol(rvalStr, &rend, 16);
@@ -149,13 +240,11 @@ bool match_vals_interface(struct Auth *rule, struct Data *d, struct udev_device
 			ret = match_valsInt(rule->devcount + 1, d->op, rval);
 	} else {
 		lvalStr = usbauth_get_param_valStr(d->param, interface); // get parameter from sysfs
+		enum Valuetype lvalType = usbauth_get_param_type(d->param);
 		if (lvalStr)
-			ret = match_vals(lvalStr, d->op, rvalStr);
+			ret = match_vals_autotype(lvalStr, d->op, rvalStr, lvalType);
 	}
 
-	if (tmpStr)
-		free(tmpStr);
-
 	return ret;
 }
 
@@ -226,7 +315,10 @@ bool no_error_check_dbus(DBusError *error) {
 
 	if (dbus_error_is_set(error)) {
 		ret = false;
-		syslog(LOG_ERR, "dbus_error: %s\n", error->message);
+
+		if (debuglog)
+			syslog(LOG_DEBUG, "dbus_error: %s\n", error->message);
+
 		dbus_error_free(error);
 	}
 
